@@ -105,7 +105,9 @@ class Button(BaseModel, extra="forbid"):  # type: ignore[call-arg]
         " The icon can be a URL to an image,"
         " like `'url:https://www.nijho.lt/authors/admin/avatar.jpg'`, or a `spotify:`"
         " icon, like `'spotify:album/6gnYcXVaffdG0vwVM34cr8'`."
-        " If the icon is a `spotify:` icon, the icon will be downloaded and cached.",
+        " If the icon is a `spotify:` icon, the icon will be downloaded and cached."
+        " The icon can also display a partially complete ring, like a progress bar,"
+        " or sensor value, like `ring:25` for a 25% complete ring.",
     )
     icon_mdi: str | None = Field(
         default=None,
@@ -131,13 +133,14 @@ class Button(BaseModel, extra="forbid"):  # type: ignore[call-arg]
         allow_template=False,
         description="When specifying `icon` and `entity_id`, if the state is `off`, the icon will be converted to grayscale.",
     )
-    delay: float = Field(
+    delay: float | str = Field(
         default=0.0,
-        allow_template=False,
+        allow_template=True,
         description="The delay (in seconds) before the `service` is called."
         " This is useful if you want to wait before calling the `service`."
         " Counts down from the time the button is pressed."
-        " If while counting the button is pressed again, the timer is cancelled.",
+        " If while counting the button is pressed again, the timer is cancelled."
+        " Should be a float or template string that evaluates to a float.",
     )
     special_type: Literal[
         "next-page",
@@ -146,6 +149,7 @@ class Button(BaseModel, extra="forbid"):  # type: ignore[call-arg]
         "go-to-page",
         "turn-off",
         "light-control",
+        "reload",
     ] | None = Field(
         default=None,
         allow_template=False,
@@ -158,7 +162,8 @@ class Button(BaseModel, extra="forbid"):  # type: ignore[call-arg]
         " If `go-to-page`, the button will go to the page specified by `special_type_data`"
         " (either an `int` or `str` (name of the page))."
         " If `light-control`, the button will control a light, and the `special_type_data`"
-        " can be a dictionary, see its description.",
+        " can be a dictionary, see its description."
+        " If `reload`, the button will reload the configuration file when pressed.",
     )
     special_type_data: Any | None = Field(
         default=None,
@@ -268,7 +273,7 @@ class Button(BaseModel, extra="forbid"):  # type: ignore[call-arg]
             )
             return _generate_failed_icon(size)
 
-    def render_icon(  # noqa: PLR0912
+    def render_icon(  # noqa: PLR0912 PLR0915
         self,
         complete_state: StateDict,
         *,
@@ -294,6 +299,10 @@ class Button(BaseModel, extra="forbid"):  # type: ignore[call-arg]
                 filename = _url_to_filename(id_)
                 # copy to avoid modifying the cached image
                 image = _download_image(id_, filename, size).copy()
+            if which == "ring":
+                pct = _maybe_number(id_)
+                assert isinstance(pct, (int, float)), f"Invalid ring percentage: {id_}"
+                image = _draw_percentage_ring(pct, size)
 
         icon_convert_to_grayscale = False
         text = button.text
@@ -313,6 +322,9 @@ class Button(BaseModel, extra="forbid"):  # type: ignore[call-arg]
         elif button.special_type == "turn-off":
             text = button.text or "Turn off"
             icon_mdi = button.icon_mdi or "power"
+        elif button.special_type == "reload":
+            text = button.text or "Reload\nconfig"
+            icon_mdi = button.icon_mdi or "reload"
         elif button.entity_id in complete_state:
             # Has entity_id
             state = complete_state[button.entity_id]
@@ -410,6 +422,10 @@ class Button(BaseModel, extra="forbid"):  # type: ignore[call-arg]
         """Start or cancel the timer."""
         if self.delay:
             if self._timer is None:
+                assert isinstance(
+                    self.delay,
+                    (int, float),
+                ), f"Invalid delay: {self.delay}"
                 self._timer = AsyncDelayedCallback(delay=self.delay, callback=callback)
             if self._timer.is_running():
                 self._timer.cancel()
@@ -428,6 +444,7 @@ class Button(BaseModel, extra="forbid"):  # type: ignore[call-arg]
     ) -> tuple[Button, Image.Image]:
         """Return the button and image for the sleep button."""
         assert self._timer is not None
+        assert isinstance(self.delay, (int, float)), f"Invalid delay: {self.delay}"
         remaining = self._timer.remaining_time()
         pct = round(remaining / self.delay * 100)
         image = _draw_percentage_ring(pct, size)
@@ -529,6 +546,22 @@ class Config(BaseModel):
     _current_page_index: int = PrivateAttr(default=0)
     _is_on: bool = PrivateAttr(default=True)
     _detached_page: Page | None = PrivateAttr(default=None)
+    _configuration_file: Path | None = PrivateAttr(default=None)
+
+    @classmethod
+    def load(cls: type[Config], fname: Path) -> Config:
+        """Read the configuration file."""
+        with fname.open() as f:
+            data = yaml.safe_load(f)
+            config = cls(**data)
+            config._configuration_file = fname
+            return config
+
+    def reload(self) -> None:
+        """Reload the configuration file."""
+        assert self._configuration_file is not None
+        new = self.load(self._configuration_file).__dict__
+        self.__dict__.update(new)
 
     @classmethod
     def to_pandas_table(cls: type[Config]) -> pd.DataFrame:
@@ -680,7 +713,7 @@ class AsyncDelayedCallback:
 
 
 def _draw_percentage_ring(
-    percentage: int,
+    percentage: int | float,
     size: tuple[int, int],
     *,
     radius: int | None = None,
@@ -954,7 +987,9 @@ def _state_attr(
     complete_state: StateDict,
 ) -> Any:
     """Get the state attribute for an entity."""
-    return complete_state.get(entity_id, {}).get("attributes", {}).get(attr)
+    attrs = complete_state.get(entity_id, {}).get("attributes", {})
+    state_attr = attrs.get(attr)
+    return _maybe_number(state_attr)
 
 
 def _is_state_attr(
@@ -964,7 +999,43 @@ def _is_state_attr(
     complete_state: StateDict,
 ) -> bool:
     """Check if the state attribute for an entity is a value."""
-    return _state_attr(entity_id, attr, complete_state) == value
+    return _state_attr(entity_id, attr, complete_state) == _maybe_number(value)
+
+
+def _is_float(s: str) -> bool:
+    try:
+        float(s)
+    except ValueError:
+        return False
+    else:
+        return True
+
+
+def _maybe_number(s: str, *, rounded: bool = False) -> int | str | float:
+    """Convert a string to a number if possible."""
+    if not isinstance(s, str):  # already a number or other type
+        return s
+
+    if _is_integer(s):
+        num = int(s)
+    elif _is_float(s):
+        num = float(s)  # type: ignore[assignment]
+    else:
+        return s
+
+    if rounded:
+        return round(num)
+
+    return num
+
+
+def _is_integer(s: str) -> bool:
+    try:
+        int(s)
+    except ValueError:
+        return False
+    else:
+        return True
 
 
 def _states(
@@ -980,12 +1051,11 @@ def _states(
     if not entity_state:
         return None
     state = entity_state["state"]
+    state = _maybe_number(state, rounded=rounded)
     if with_unit:
         unit = entity_state.get("attributes", {}).get("unit_of_measurement")
         if unit:
             state += f" {unit}"
-    if rounded:
-        state = round(float(state))
     return state
 
 
@@ -995,7 +1065,7 @@ def _is_state(
     complete_state: StateDict,
 ) -> bool:
     """Check if the state for an entity is a value."""
-    return _states(entity_id, complete_state=complete_state) == state
+    return _states(entity_id, complete_state=complete_state) == _maybe_number(state)
 
 
 def _min_filter(value: float, other_value: float) -> float:
@@ -1260,13 +1330,6 @@ def get_deck() -> StreamDeck:
     return deck
 
 
-def read_config(fname: Path) -> Config:
-    """Read the configuration file."""
-    with fname.open() as f:
-        data = yaml.safe_load(f)
-        return Config(**data)
-
-
 def turn_on(config: Config, deck: StreamDeck, complete_state: StateDict) -> None:
     """Turn on the Stream Deck and update all key images."""
     console.log(f"Calling turn_on, with {config._is_on=}")
@@ -1329,6 +1392,10 @@ async def _handle_key_press(
         config._detached_page = page
         update_all()
         return  # to skip the _detached_page reset below
+    elif button.special_type == "reload":
+        config.reload()
+        update_all()
+        return
     elif button.service is not None:
         button = button.rendered_template_button(complete_state)
         if button.service_data is None:
@@ -1666,7 +1733,7 @@ def main() -> None:
     console.log(
         f"Starting Stream Deck integration with {args.host=}, {args.config=}, {args.protocol=}",
     )
-    config = read_config(args.config)
+    config = Config.load(args.config)
     asyncio.run(
         run(
             host=args.host,
